/*
 * moda.c - test module for kmod_def_cleanup() api
 *
 * Copyright (c) International Business Machines Corp., 2000
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Maneesh Soni <smaneesh@in.ibm.com>
 *
 */                          

#include "mod.h"

shared_list_t lista;
int node_count = 0;

spinlock_t listlock = SPIN_LOCK_UNLOCKED;

void readlist(void)
{
	struct list_head * p;

	spin_lock(&listlock);
	list_for_each(p, &(lista.link)) ;
	spin_unlock(&listlock);

	return;
}

void writelist(char *arg)
{
	shared_list_t *new;
	struct list_head *p;

	if (!(strcmp(arg, "add"))) {
		if ((new = (shared_list_t *) kmalloc(sizeof(*new), 
						      GFP_ATOMIC))) {
			new->data = node_count++;
			spin_lock(&listlock);
			list_add(&(new->link), &(lista.link));
			spin_unlock(&listlock);
		}
	}
	else if (!(strcmp(arg, "del"))) {
		spin_lock(&listlock);
		list_for_each(p, &(lista.link)) {
			if (((shared_list_t *)p)->data == (--node_count)) {
				list_del(p);
			}
		spin_unlock(&listlock);
		}
	}
	return;

}

static int moda_open(struct inode *i, struct file *f)
{
	if (!try_inc_mod_count(THIS_MODULE))
		return -ENODEV;
	return 0;
}

static int moda_release(struct inode *i, struct file *f)
{
	MOD_DEC_USE_COUNT;

	return 0;
}

static  ssize_t moda_write(struct file * f, const char * buf, size_t s, 
			    loff_t * o)
{
	char *arg;

	arg = (char *) kmalloc(s, GFP_KERNEL);
	if (copy_from_user(arg, buf, s))
                return -EFAULT;   
	writelist(arg);

	return 0;
}

static ssize_t moda_read(struct file * f, char * c, size_t s, loff_t * o)
{
	readlist();

	return 0;
}

static struct file_operations moda_fops = {
  owner:        THIS_MODULE,
  open:		moda_open,
  read:         moda_read,
  write:	moda_write,
  release:	moda_release
};    


void del_list(shared_list_t *l)
{
	struct list_head * p;

	printk("moda del_list: deleteing internal data\n");

	list_for_each(p, &(l->link)) {
		list_del(p);
		kfree(p);
	}
	return ;
}

static module_data_destructor_t mdd;

void cleanup_module(void)
{

	printk("moda: cleanup_module\n");

	if (unregister_chrdev(MODA_MAJOR, "moda") < 0) 
		printk(KERN_ERR "moda: unregister failed\n");

	mdd.func = (module_data_destructor_func_t) del_list;
	mdd.arg = &lista;
	kmod_def_cleanup(THIS_MODULE, &mdd);
		
}

int init_module(void)
{
	printk("moda: init_module\n");

	INIT_LIST_HEAD(&lista.link);

 	if (register_chrdev(MODA_MAJOR, "moda", &moda_fops)) {
    		printk(KERN_ERR "moda: unable to get major %d for moda\n",
           		MODA_MAJOR);
    		return -EBUSY;
  	}       

	return 0;
}
