
/*
 * moda.c - test module for kmod_def_cleanup() api
 *
 * Copyright (c) International Business Machines Corp., 2000
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Maneesh Soni <smaneesh@in.ibm.com>
 */

#include "mod.h"

extern shared_list_t lista;
extern int count;
extern void readlist(void);
extern void writelist(char *arg);


static int modb_open(struct inode *i, struct file *f)
{
	if (!try_inc_mod_count(THIS_MODULE))
		return -ENODEV;

	return 0;
}

static int modb_release(struct inode *i, struct file *f)
{
	MOD_DEC_USE_COUNT;

	return 0;
}

static  ssize_t modb_write(struct file * f, const char * buf, size_t s, 
			    loff_t * o)
{
	char *arg;

	MOD_INC_USE_COUNT;

	arg = (char *) kmalloc(s, GFP_KERNEL);

	MOD_DEC_USE_COUNT;

	if (copy_from_user(arg, buf, s))
                return -EFAULT;   
	writelist(arg);

	return 0;
}

static ssize_t modb_read(struct file * f, char * c, size_t s, loff_t * o)
{
	readlist();
	return 0;
}

static struct file_operations modb_fops = {
  owner:        THIS_MODULE,
  open:		modb_open,
  read:         modb_read,
  write:	modb_write,
  release:	modb_release
};    


void cleanup_module(void)
{
	
	printk("modb: cleanup_module\n");

	if (unregister_chrdev(MODB_MAJOR, "modb") < 0) 
		printk("modb: unregister failed\n");

	kmod_def_cleanup(THIS_MODULE, NULL);

}

int init_module(void)
{
	printk("modb: init_module\n");
 	if (register_chrdev(MODB_MAJOR, "modb", &modb_fops)) {
    		printk(KERN_ERR "modb: unable to get major %d for modb\n",
           		MODB_MAJOR);
    		return -EBUSY;
  	}       

	return 0;
}
